package com.bpcl.lpg.core.dao.impl;

import org.apache.log4j.LogManager;
import com.bpcl.lpg.core.model.AlternateNumberOtpModel;
import de.hybris.platform.servicelayer.search.paginated.util.PaginatedSearchUtils;
import de.hybris.platform.servicelayer.search.paginated.PaginatedFlexibleSearchParameter;
import de.hybris.platform.core.servicelayer.data.SearchPageData;
import com.bpcl.lpg.core.model.CashMemoFileTrackerModel;
import java.util.Date;
import com.bpcl.lpg.core.model.LpgUndeliveredReasonModel;
import java.util.Collections;
import java.util.Objects;
import com.bpcl.lpg.core.model.CashMemoModel;
import de.hybris.platform.servicelayer.exceptions.SystemException;
import de.hybris.platform.servicelayer.search.SearchResult;
import de.hybris.platform.servicelayer.exceptions.ModelNotFoundException;
import de.hybris.platform.servicelayer.search.exceptions.FlexibleSearchException;
import java.util.Collection;
import org.springframework.util.CollectionUtils;
import java.util.Arrays;
import java.util.Map;
import de.hybris.platform.servicelayer.search.FlexibleSearchQuery;
import java.util.HashMap;
import java.util.List;
import de.hybris.platform.servicelayer.search.paginated.PaginatedFlexibleSearchService;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;
import org.apache.log4j.Logger;
import com.bpcl.lpg.core.dao.LpgDeliveryConfirmationDao;

public class LpgDeliveryConfirmationDaoImpl implements LpgDeliveryConfirmationDao
{
    private static final Logger LOG;
    private static final String PERCENT = "%";
    private static final int ZERO = 0;
    private FlexibleSearchService flexibleSearchService;
    private PaginatedFlexibleSearchService paginatedFlexibleSearchService;
    private static final String GET_CASH_MEMO_DETAILS = "SELECT {pk} FROM {CashMemo} WHERE {distCode}=?distCode AND {refillId}=?refillId AND {dayEnd}=?dayEnd AND {action}=?action AND {sentToLPGNxt}=0";
    private static final String GET_DIST_CASH_MEMO_DETAILS = "SELECT {pk} FROM {CashMemo} WHERE {distCode}=?distCode AND {dayEnd}=?dayEnd";
    private static final String GET_CASH_MEMO_DETAILS_FOR_SENDING_OTP = "SELECT {pk} FROM {CashMemo} WHERE {otpSent}=?otpSent AND {dayEnd}=?dayEnd AND {deliveryStatus}=?deliveryStatus AND {creationTime}>?startDate AND {creationTime}<?endDate";
    private static final String GET_OTP_COUNT_BY_NUMBER = "SELECT {pk} FROM {AlternateNumberOtp} WHERE {mobileNumber}=?mobileNumber";
    private static final String GET_CASH_MEMO_DETAILS_BY_REFILL = "SELECT {pk} FROM {CashMemo} WHERE {refillId}=?refillId";
    
    public List<Object> getOrderCountByAreaId(final String distCode, final String staffId) {
        final StringBuilder query = new StringBuilder("SELECT {areaId} as AreaCode, ").append("{area} as AreaName, ").append("COUNT({orderNo}) as NoOfOrders ").append("from {").append("CashMemo").append("} WHERE {").append("distCode").append("}=?").append("distcode").append(" AND {").append("deliveryBoy").append("}= ?").append("staffid").append(" AND {").append("action").append("}= 'A' ").append(" AND {").append("dayEnd").append("}= 0 ").append(" AND {").append("deliveryStatus").append("}= 0 ").append(" GROUP By {areaId},{area}");
        final Map<String, Object> param = new HashMap<String, Object>();
        param.put("distcode", distCode);
        param.put("staffid", staffId);
        final FlexibleSearchQuery flexibleSearchQuery = new FlexibleSearchQuery(query);
        flexibleSearchQuery.addQueryParameters((Map)param);
        flexibleSearchQuery.setResultClassList((List)Arrays.asList(String.class, String.class, Integer.class));
        try {
            final SearchResult<Object> searchResult = (SearchResult<Object>)this.getFlexibleSearchService().search(flexibleSearchQuery);
            return CollectionUtils.isEmpty((Collection)searchResult.getResult()) ? null : searchResult.getResult();
        }
        catch (final FlexibleSearchException | ModelNotFoundException ex) {
            LpgDeliveryConfirmationDaoImpl.LOG.error((Object)"Cash Memo Data not found Error ", (Throwable)ex);
            return null;
        }
    }
    
    public List<CashMemoModel> getCashMemoByStaffId(final String staffId, final String distCode, final String areaCode, final String consumerName, final String consumerNo, final String customerMobile) {
        final Map<String, Object> param = new HashMap<String, Object>();
        final StringBuilder query = new StringBuilder("SELECT {pk} from {").append("CashMemo").append("} WHERE {").append("distCode").append("}=?").append("distcode").append(" AND {").append("deliveryBoy").append("}= ?").append("staffid").append(" AND {").append("action").append("}= 'A' ").append(" AND {").append("dayEnd").append("}= 0 ").append("AND {").append("deliveryStatus").append("}= 0 ");
        if (areaCode != null) {
            query.append(" AND {").append("areaId").append("} = ?").append("areacode");
            param.put("areacode", areaCode);
        }
        if (consumerName != null) {
            query.append(" AND {").append("consumerName").append("} like ?").append("consumername");
            final String customerName = "%".concat(consumerName).concat("%");
            param.put("consumername", customerName);
        }
        if (customerMobile != null) {
            query.append(" AND {").append("mobileNumber").append("} = ?").append("customermobile");
            param.put("customermobile", customerMobile);
        }
        if (consumerNo != null) {
            query.append(" AND {").append("consumerNumber").append("} = ?").append("consumernumber");
            param.put("consumernumber", consumerNo);
        }
        param.put("distcode", distCode);
        param.put("staffid", staffId);
        final FlexibleSearchQuery flexibleSearchQuery = new FlexibleSearchQuery(query);
        flexibleSearchQuery.addQueryParameters((Map)param);
        final SearchResult<CashMemoModel> search = (SearchResult<CashMemoModel>)this.getFlexibleSearchService().search(flexibleSearchQuery);
        return Objects.isNull(search) ? Collections.emptyList() : search.getResult();
    }
    
    public CashMemoModel getCashMemoDetails(final String cashMemoSerialNumber) {
        final StringBuilder query = new StringBuilder("SELECT {pk} from {").append("CashMemo").append("} WHERE {").append("cashMemoSerialNumber").append("}= ?").append("cashMemoSerialNumber");
        final Map<String, Object> param = new HashMap<String, Object>();
        param.put("cashMemoSerialNumber", cashMemoSerialNumber);
        final FlexibleSearchQuery flexibleSearchQuery = new FlexibleSearchQuery(query);
        flexibleSearchQuery.addQueryParameters((Map)param);
        final SearchResult<CashMemoModel> search = (SearchResult<CashMemoModel>)this.getFlexibleSearchService().search(flexibleSearchQuery);
        return search.getResult().get(0);
    }
    
    public List<LpgUndeliveredReasonModel> getUndeliveredReasonCodes() {
        final StringBuilder query = new StringBuilder("SELECT {pk} from {").append("LpgUndeliveredReason").append("}");
        final FlexibleSearchQuery flexibleSearchQuery = new FlexibleSearchQuery(query);
        final SearchResult<LpgUndeliveredReasonModel> search = (SearchResult<LpgUndeliveredReasonModel>)this.getFlexibleSearchService().search(flexibleSearchQuery);
        return Objects.isNull(search) ? Collections.emptyList() : search.getResult();
    }
    
    public CashMemoFileTrackerModel getCashMemoFilesData(final Date startDate, final Date endDate) {
        final StringBuilder query = new StringBuilder("SELECT {pk} from {").append("CashMemoFileTracker").append("} WHERE {").append("processStartTimeStamp").append("} > ?").append("startDate").append(" AND {").append("processStartTimeStamp").append("} < ?").append("endDate").append(" ORDER BY {").append("processStartTimeStamp").append("} DESC");
        final Map<String, Object> param = new HashMap<String, Object>();
        param.put("startDate", startDate);
        param.put("endDate", endDate);
        final FlexibleSearchQuery flexibleSearchQuery = new FlexibleSearchQuery(query);
        flexibleSearchQuery.addQueryParameters((Map)param);
        final SearchResult<CashMemoFileTrackerModel> cashMemoFiles = (SearchResult<CashMemoFileTrackerModel>)this.getFlexibleSearchService().search(flexibleSearchQuery);
        if (cashMemoFiles.getResult().isEmpty()) {
            throw new ModelNotFoundException("No Cash memo files in last few hours found");
        }
        return cashMemoFiles.getResult().get(0);
    }
    
    public SearchPageData<CashMemoModel> getCashMemoDetailsForExport(final Date startDate, final Date endDate, final int pageSize, final int currentPage) {
        final StringBuilder query = new StringBuilder("SELECT {pk} from {").append("CashMemo").append("} WHERE {").append("deliveryStatus").append("} IN (1,2) ").append(" AND {").append("modifiedtime").append("} > ?").append("startDate").append(" AND {").append("modifiedtime").append("} < ?").append("endDate").append(" AND {").append("sentToLPGNxt").append("} = 0");
        final Map<String, Object> param = new HashMap<String, Object>();
        param.put("startDate", startDate);
        param.put("endDate", endDate);
        LpgDeliveryConfirmationDaoImpl.LOG.info("getCashMemoDetailsForExport query : " + query.toString());
        final FlexibleSearchQuery flexibleSearchQuery = new FlexibleSearchQuery(query);
        flexibleSearchQuery.addQueryParameters((Map)param);
        return (SearchPageData<CashMemoModel>)this.paginatedFlexibleSearchService.search(this.createPageableData(flexibleSearchQuery, pageSize, currentPage));
    }
    
    public SearchPageData<CashMemoModel> getCashMemoDetailsForSendingOTP(final String startDate, final String endDate, final int pageSize, final int currentPage) {
        final FlexibleSearchQuery flexibleSearchQuery = new FlexibleSearchQuery("SELECT {pk} FROM {CashMemo} WHERE {otpSent}=?otpSent AND {dayEnd}=?dayEnd AND {deliveryStatus}=?deliveryStatus AND {creationTime}>?startDate AND {creationTime}<?endDate");
        flexibleSearchQuery.addQueryParameter("otpSent", (Object)0);
        flexibleSearchQuery.addQueryParameter("dayEnd", (Object)0);
        flexibleSearchQuery.addQueryParameter("deliveryStatus", (Object)0);
        flexibleSearchQuery.addQueryParameter("startDate", (Object)startDate);
        flexibleSearchQuery.addQueryParameter("endDate", (Object)endDate);
        return (SearchPageData<CashMemoModel>)this.paginatedFlexibleSearchService.search(this.createPageableData(flexibleSearchQuery, pageSize, currentPage));
    }
    
    private PaginatedFlexibleSearchParameter createPageableData(final FlexibleSearchQuery query, final int batchSize, final int currentPage) {
        final SearchPageData<CashMemoModel> searchPageData = (SearchPageData<CashMemoModel>)PaginatedSearchUtils.createSearchPageDataWithPagination(batchSize, currentPage, (boolean)Boolean.TRUE);
        final PaginatedFlexibleSearchParameter parameter = new PaginatedFlexibleSearchParameter();
        parameter.setFlexibleSearchQuery(query);
        parameter.setSearchPageData((SearchPageData)searchPageData);
        return parameter;
    }
    
    public CashMemoModel getCashMemoByRefillId(final int distCode, final String refillId, final boolean dayEnd) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {CashMemo} WHERE {distCode}=?distCode AND {refillId}=?refillId AND {dayEnd}=?dayEnd AND {action}=?action AND {sentToLPGNxt}=0");
        query.addQueryParameter("distCode", (Object)distCode);
        query.addQueryParameter("refillId", (Object)refillId);
        query.addQueryParameter("dayEnd", (Object)dayEnd);
        query.addQueryParameter("action", (Object)'A');
        final SearchResult<CashMemoModel> cashMemoModelSearchResult = (SearchResult<CashMemoModel>)this.flexibleSearchService.search(query);
        return cashMemoModelSearchResult.getResult().isEmpty() ? null : cashMemoModelSearchResult.getResult().get(0);
    }
    
    public List<CashMemoModel> getCashMemoByDistId(final int distCode, final boolean dayEnd) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {CashMemo} WHERE {distCode}=?distCode AND {dayEnd}=?dayEnd");
        query.addQueryParameter("distCode", (Object)distCode);
        query.addQueryParameter("dayEnd", (Object)dayEnd);
        final SearchResult<CashMemoModel> cashMemoModelSearchResult = (SearchResult<CashMemoModel>)this.flexibleSearchService.search(query);
        return cashMemoModelSearchResult.getResult();
    }
    
    public AlternateNumberOtpModel getOtpCountByMobileNumber(final String mobileNumber) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {AlternateNumberOtp} WHERE {mobileNumber}=?mobileNumber");
        query.addQueryParameter("mobileNumber", (Object)mobileNumber);
        final SearchResult<AlternateNumberOtpModel> searchResults = (SearchResult<AlternateNumberOtpModel>)this.flexibleSearchService.search(query);
        return searchResults.getResult().isEmpty() ? null : searchResults.getResult().get(0);
    }
    
    public CashMemoModel getCashMemoByRefillId(final String refillId) {
        final FlexibleSearchQuery query = new FlexibleSearchQuery("SELECT {pk} FROM {CashMemo} WHERE {refillId}=?refillId");
        query.addQueryParameter("refillId", (Object)refillId);
        final SearchResult<CashMemoModel> cashMemoModelSearchResult = (SearchResult<CashMemoModel>)this.flexibleSearchService.search(query);
        return cashMemoModelSearchResult.getResult().isEmpty() ? null : cashMemoModelSearchResult.getResult().get(0);
    }
    
    public FlexibleSearchService getFlexibleSearchService() {
        return this.flexibleSearchService;
    }
    
    public void setFlexibleSearchService(final FlexibleSearchService flexibleSearchService) {
        this.flexibleSearchService = flexibleSearchService;
    }
    
    public void setPaginatedFlexibleSearchService(final PaginatedFlexibleSearchService paginatedFlexibleSearchService) {
        this.paginatedFlexibleSearchService = paginatedFlexibleSearchService;
    }
    
    static {
        LOG = LogManager.getLogger((Class)LpgDeliveryConfirmationDaoImpl.class);
    }
}
